# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-07 05:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0006_rawdatafile_error_log_archive'),
    ]

    operations = [
        migrations.AddField(
            model_name='rawdatafile',
            name='error_count',
            field=models.IntegerField(default=0, help_text='Count of records in the original download that could not be parsed and are excluded from the cleaned file.', verbose_name='error count'),
        ),
    ]
